'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

// Tokyo Night Storm theme color definitions
const base00 = '#24283b', // Background
base01 = '#7982a9', // Foreground
base02 = '#6f7bb630', // Selection Match
base03 = '#565f89', // Comment
base04 = '#c0caf5', // Cursor, Variable Name, and several others
base05 = '#c0caf5', // Light blue for brighter foreground
base06 = '#292e427a', // Line Highlight
base07 = '#bb9af7', // Keyword, Operator, Constant
base08 = '#7aa2f7', // Property Name, Function
base09 = '#9ece6a', // Strings
base0A = '#ff9e64', // Numbers
base0B = '#2ac3de', // Type Name
base0C = '#b4f9f8', // Regexp
base0D = '#89ddff', // Heading
base0E = '#ff5370', // Invalid
base0F = '#ff5370'; // Link
const invalid = '#ff5370', // Bright red for errors
darkBackground = base00, highlightBackground = base06, background = base00, // Main background
tooltipBackground = base01, selection = base02, cursor = base04; // Cursor color
/**
The editor theme styles for Tokyo Night Storm.
*/
const tokyoNightStormTheme = view.EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: background,
    },
    '.cm-content': {
        caretColor: cursor,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base03,
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: '2px solid black',
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: '2px solid black',
    },
    '.cm-searchMatch': {
        backgroundColor: '#7aa2f730', // Lighter shade of property name color
        outline: `1px solid ${base03}`,
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05,
    },
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
    },
    '.cm-selectionMatch': {
        backgroundColor: base05,
    },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        outline: `1px solid ${base03}`,
    },
    '&.cm-focused .cm-matchingBracket': {
        backgroundColor: base06,
    },
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03,
        border: 'none',
    },
    '.cm-activeLineGutter': {
        backgroundColor: highlightBackground,
    },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: base05,
    },
    '.cm-tooltip': {
        border: 'none',
        backgroundColor: tooltipBackground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            backgroundColor: highlightBackground,
            color: base03,
        },
    },
}, { dark: true });
/**
The highlighting style for code in the Tokyo Night Storm theme.
*/
const tokyoNightStormHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: base07 },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base04 },
    { tag: [highlight.tags.propertyName], color: base08 },
    {
        tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted, highlight.tags.special(highlight.tags.string)],
        color: base09,
    },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: base08 },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base07 },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base04 },
    { tag: [highlight.tags.className], color: base04 },
    {
        tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace],
        color: base0A,
    },
    {
        tag: [highlight.tags.annotation],
        color: invalid,
    },
    { tag: [highlight.tags.typeName], color: base0B },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base07 },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.regexp, highlight.tags.link], color: base0C },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: base03 },
    { tag: highlight.tags.tagName, color: base07 },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.link, color: base0F, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: base0D },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base04 },
    { tag: highlight.tags.invalid, color: base0E },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
]);
/**
Extension to enable the Tokyo Night Storm theme (both the editor theme and the highlight style).
*/
const tokyoNightStorm = [
    tokyoNightStormTheme,
    language.syntaxHighlighting(tokyoNightStormHighlightStyle),
];

exports.tokyoNightStorm = tokyoNightStorm;
exports.tokyoNightStormHighlightStyle = tokyoNightStormHighlightStyle;
exports.tokyoNightStormTheme = tokyoNightStormTheme;
