import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';

// Tokyo Night Storm theme color definitions
const base00 = '#24283b', // Background
base01 = '#7982a9', // Foreground
base02 = '#6f7bb630', // Selection Match
base03 = '#565f89', // Comment
base04 = '#c0caf5', // Cursor, Variable Name, and several others
base05 = '#c0caf5', // Light blue for brighter foreground
base06 = '#292e427a', // Line Highlight
base07 = '#bb9af7', // Keyword, Operator, Constant
base08 = '#7aa2f7', // Property Name, Function
base09 = '#9ece6a', // Strings
base0A = '#ff9e64', // Numbers
base0B = '#2ac3de', // Type Name
base0C = '#b4f9f8', // Regexp
base0D = '#89ddff', // Heading
base0E = '#ff5370', // Invalid
base0F = '#ff5370'; // Link
const invalid = '#ff5370', // Bright red for errors
darkBackground = base00, highlightBackground = base06, background = base00, // Main background
tooltipBackground = base01, selection = base02, cursor = base04; // Cursor color
/**
The editor theme styles for Tokyo Night Storm.
*/
const tokyoNightStormTheme = /*@__PURE__*/EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: background,
    },
    '.cm-content': {
        caretColor: cursor,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base03,
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: '2px solid black',
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: '2px solid black',
    },
    '.cm-searchMatch': {
        backgroundColor: '#7aa2f730', // Lighter shade of property name color
        outline: `1px solid ${base03}`,
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05,
    },
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
    },
    '.cm-selectionMatch': {
        backgroundColor: base05,
    },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        outline: `1px solid ${base03}`,
    },
    '&.cm-focused .cm-matchingBracket': {
        backgroundColor: base06,
    },
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03,
        border: 'none',
    },
    '.cm-activeLineGutter': {
        backgroundColor: highlightBackground,
    },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: base05,
    },
    '.cm-tooltip': {
        border: 'none',
        backgroundColor: tooltipBackground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            backgroundColor: highlightBackground,
            color: base03,
        },
    },
}, { dark: true });
/**
The highlighting style for code in the Tokyo Night Storm theme.
*/
const tokyoNightStormHighlightStyle = /*@__PURE__*/HighlightStyle.define([
    { tag: tags.keyword, color: base07 },
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: base04 },
    { tag: [tags.propertyName], color: base08 },
    {
        tag: [tags.processingInstruction, tags.string, tags.inserted, /*@__PURE__*/tags.special(tags.string)],
        color: base09,
    },
    { tag: [/*@__PURE__*/tags.function(tags.variableName), tags.labelName], color: base08 },
    { tag: [tags.color, /*@__PURE__*/tags.constant(tags.name), /*@__PURE__*/tags.standard(tags.name)], color: base07 },
    { tag: [/*@__PURE__*/tags.definition(tags.name), tags.separator], color: base04 },
    { tag: [tags.className], color: base04 },
    {
        tag: [tags.number, tags.changed, tags.modifier, tags.self, tags.namespace],
        color: base0A,
    },
    {
        tag: [tags.annotation],
        color: invalid,
    },
    { tag: [tags.typeName], color: base0B },
    { tag: [tags.operator, tags.operatorKeyword], color: base07 },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: base0C },
    { tag: [tags.meta, tags.comment], color: base03 },
    { tag: tags.tagName, color: base07 },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, color: base0F, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: base0D },
    { tag: [tags.atom, tags.bool, /*@__PURE__*/tags.special(tags.variableName)], color: base04 },
    { tag: tags.invalid, color: base0E },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
]);
/**
Extension to enable the Tokyo Night Storm theme (both the editor theme and the highlight style).
*/
const tokyoNightStorm = [
    tokyoNightStormTheme,
    /*@__PURE__*/syntaxHighlighting(tokyoNightStormHighlightStyle),
];

export { tokyoNightStorm, tokyoNightStormHighlightStyle, tokyoNightStormTheme };
